/**
 * This class represents Monty in the Monty Hall simulation.
 * This is the main control class for the simulation.
 *
 * @author Nathaniel Titterton
 * @version 0.2
 */

import java.util.Random;
import java.util.Date;

public class Monty {


   // do we need this here?  do we want it here?
   public static void main(String[] args) {
	    Monty monty = new Monty();
       monty.play();
       }


   public Display theDisplay = new Display();
   public Player thePlayer = new Player(theDisplay);


   /**
    * Constructor for Monty
    */
   public Monty() {
      theDisplay.buildTheStage();
   }


   public boolean play() {
       int doorWithCar;
       int initialChoice;
       int revealedDoor;
       int finalChoice;

       theDisplay.setupNewGame();
       thePlayer.setDisplay(theDisplay);

       doorWithCar = placeCar();

       initialChoice = getPlayersInitialChoice();

       revealedDoor = revealGoat(initialChoice, doorWithCar);   // need better method name!

       finalChoice = getPlayersFinalChoice(initialChoice, revealedDoor);

       describeResults(finalChoice, doorWithCar);

       return(finalChoice == doorWithCar);
   }


   private int placeCar() {
       // randomly set doorWithCar to be 1, 2, or 3
       int doorWithCar = chooseRandomly(1, 2, 3);
       theDisplay.placeCar(doorWithCar);
       return(doorWithCar);
   }


   private int getPlayersInitialChoice() {
       int choice = 0;

       theDisplay.setDoorsSelectableByPlayer(thePlayer);
       say("Pick a door, any door.");
       while (choice < 1 || choice > 3) {
           choice = thePlayer.chooseInitialDoor();
       }
       theDisplay.setDoorsUnselectable();

       theDisplay.markInitialChoice(choice);
       return choice;
   }



   private int revealGoat(int initialChoice, int doorWithCar) {
       int reveal;

       if (initialChoice == doorWithCar) {
           // the contestant chose the car door; so, reveal one of the other two randomly.
           if (doorWithCar == 1)
               reveal = chooseRandomly(2, 3);
           else if (doorWithCar == 2)
               reveal = chooseRandomly(1, 3);
           else {  // initialChoice and carDoor are both 3
               reveal = chooseRandomly(1, 2);
           }
       }
       else {
           // the contestant didn't choose the car door, so reveal the door
           // that is neither the contestants choice nor the car door.
           reveal = otherDoor(initialChoice, doorWithCar);
       }
       theDisplay.openDoor(reveal);
       return(reveal);
   }



   private int getPlayersFinalChoice(int initialChoice, int revealedDoor) {

       String finalChoice = "";
       theDisplay.displaySwitchOrStayOptionsForPlayer(thePlayer);
       say("You picked door " + initialChoice +
       ". I'll show you a goat. Do you want to switch or stick with your original choice?");

       while (!finalChoice.equals("switch") && !finalChoice.equals("stay")) {
           // we'll stay in this loop until we can get a return value...
           finalChoice = thePlayer.switchOrStay(initialChoice, revealedDoor);
       }
       theDisplay.hideSwitchOrStayOptions();
       if (finalChoice.equals("switch")) {          // they want to switch
           return(otherDoor(initialChoice, revealedDoor));
       } else {     // they want to stay
           return(initialChoice);
       }
   }


  // tell them if them won or not
   private void describeResults(int finalChoice, int doorWithCar) {
       theDisplay.openDoor(finalChoice);
       if (finalChoice == doorWithCar) {
    	   theDisplay.celebrate();
           say("Woot! You got the car!");
       } else {
    	   theDisplay.mourn();
           say("Sorry, you have a goat now.");
       }
   }



   //////////////////////////////////////////////////////////////////////
   // helper methods
   //   these methods are used by those above to work correctly, but
   //   don't represent specific actions in the monty hall game
   //
   //////////////////////////////////////////////////////////////////////

   // when given two doors (represented as ints) returns the number of the
   // third door.  Work through a table of possibilities to see how this works.
   private int otherDoor(int door1, int door2) {
       return (6 - (door1 + door2));
   }


   // Write something to the graphics window
   private void say(String message) {
       theDisplay.say(message);
   }


   // initializes a random generator object
   private Random randomGenerator = new Random(new Date().getTime());

   // we won't have arrays yet, will we...
   // chooses one of the two integers and returns it.
   private int chooseRandomly(int choiceA, int choiceB) {
       int aRandomNumber = randomGenerator.nextInt(2);  // returns 0 or 1
       if (aRandomNumber == 0) {
           return(choiceA);
       } else {
           return(choiceB);
       }
   }

   // chooses one of the three integers and returns it.
   private int chooseRandomly(int choiceA, int choiceB, int choiceC) {
       int aRandomNumber = randomGenerator.nextInt(3);  // returns 0, 1, or 2
       if (aRandomNumber == 0) {
           return(choiceA);
       } else if (aRandomNumber == 1) {
           return(choiceB);
       } else {
           return(choiceC);
       }
   }


}
